#                                                         -*- Perl -*-
# Copyright (c) 1999, 2000  Motoyuki Kasahara
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

#
# ñ᤿ե륯饹 (FreeUWING::Word, 
# FreeUWING::EndWord) Τδ첾ۥ饹
#
package FreeUWING::BaseWord;

require 5.005;
require Exporter;
use FileHandle;
use English;
use strict;
use integer;
use FreeUWING::Table;

use vars qw(@ISA
	    @EXPORT
	    @EXPORT_OK
	    $max_word_length
	    $word_direction
	    $endword_direction);
#	    %normalization_table);

@ISA = qw(Exporter);

#
# ñκĹ
#
$max_word_length = 255;

#
# ñν񤭹
#
$word_direction = 1;
$endword_direction = -1;

#
# :
#	new()
# ᥽åɤζʬ:
# 	public 饹᥽åɡ
# :
# 	֥Ȥ롣
# :
# 	֥ȤؤΥե󥹤֤
#
sub new {
    my $type = shift;
    my $new = {
	# ñեΥϥɥ
	'handle' => FileHandle->new(),

	# ñե̾
	'file_name' => '',

	# ޤǤ˽񤭹ȥ
	'entry_count' => 0,

	# ñν񤭹 ($word_direction  $endword_direction)
	'direction' => $word_direction,

	# 顼å
	'error_message' => '',
    };
    return bless($new, $type);
}

#
# :
#	open(file_name)
#           file_name
#		ñե̾
# ᥽åɤζʬ:
# 	public 󥹥󥹥᥽å
# :
# 	񤭹Ѥñե򳫤
# :
#	 1 ֤Ԥ 0 ֤
#
sub open {
    my $self = shift;
    my ($file_name) = @ARG;

    #
    # ñե򳫤
    #
    $self->{'file_name'} = $file_name;
    if (!$self->{'handle'}->open($self->{'file_name'}, 'w')) {
	$self->{'error_message'} =
	    "failed to open the file, $ERRNO: " . $self->{'file_name'};
	$self->close_internal();
	return 0;
    }

    return 1;
}

#
# :
#	close()
# ᥽åɤζʬ:
# 	public 󥹥󥹥᥽åɡ
# :
# 	ñեĤ롣ƤʤС⤷ʤ
# :
#	 1 ֤
#
sub close {
    my $self = shift;

    $self->close_internal();
    return 1;
}

#
# :
#	close_internal()
# ᥽åɤζʬ:
# 	private 󥹥󥹥᥽åɡ
# :
#       close() ѥ᥽åɡ
#
sub close_internal {
    my $self = shift;

    if ($self->{'handle'}->fileno()) {
	$self->{'handle'}->close();
    }
}

#
# :
#	add_entry(word, heading_position, heading_file_name,
#		  text_position, text_file_name)
#           word
#		ñ
#           heading_position
# 		Фΰ
#           heading_file_name
# 		ФΥե̾
#           text_position
# 		ʸΰ
#           text_file_name
# 		ʸΥե̾
# ᥽åɤζʬ:
# 	public 󥹥󥹥᥽åɡ
# :
# 	ñեñɲä롣
# :
#	 1 ֤Ԥ 0 ֤
#
sub add_entry {
    my $self = shift;
    my ($word, $heading_position, $heading_file_name, $text_position,
	$text_file_name) = @ARG;
    my $tmp;

    #
    # ñ롣
    #
    $word =~ s/[\x00-\x20]+//g;
    if (length($word) == 0) {
	$self->{'error_message'} = "word is empty";
	$self->close_internal();
	return 0;
    }

    my $fixed_word = $word;

    $tmp = '';
    while ($fixed_word =~ s/^([\x00-\x7f]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})//) {
      if (defined($utf8_normalization_table{$1})) {
	$tmp .= $utf8_normalization_table{$1};
      } else {
	$tmp .= $1;
      }
    }
    $fixed_word = $tmp;


    #
    # ñĹå롣
    #
    if (length($fixed_word) == 0) {
	$fixed_word = $word;
    }

    #
    # ǥåä顢ñҤä֤
    #
    if ($self->{'direction'} == $endword_direction) {
 	my $tmp = '';
 	while ($fixed_word =~ s/([\x00-\x7f]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})$//) {
 	    $tmp .= $1;
 	}
 	$fixed_word = $tmp;
    }
    
    #
    # ñ줬Ĺڤͤ롣
    #
    while ($max_word_length < length($fixed_word)) {
	$fixed_word =~ s/([\x00-\x7f]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})$//;
    }

    #
    # ñ쥨ȥեؽ񤭹ࡣ
    #
    if (!$self->{'handle'}
	->printf("%s\t%010x\t%s\t%010x\t%s\n", $fixed_word, $heading_position,
		 $heading_file_name, $text_position, $text_file_name)) {
	$self->{'error_message'} =
	    "failed to write the file, $ERRNO: " . $self->{'file_name'};
	$self->close_internal();
	return 0;
    }
    $self->{'entry_count'}++;

    return 1;
}

######################################################################
# <󥹥ѿ֤ͤ᥽åɷ>
#
# :
#	󥹥ѿ̾()
# ᥽åɤζʬ:
# 	public 󥹥󥹥᥽åɡ
# :
#	󥹥ѿ֤ͤ
#
sub file_name {
    my $self = shift;
    return $self->{'file_name'};
}

sub entry_count {
    my $self = shift;
    return $self->{'entry_count'};
}

sub error_message {
    my $self = shift;
    return $self->{'error_message'};
}

1;
